#!/usr/bin/python3

import logging
from argparse import ArgumentParser

import numpy as np
import zarr

logging.basicConfig(level=logging.INFO)


def check_interpolation(source_var, target_var):
    logging.info(f"Checking {source_var.name} to {target_var.name}")
    if target_var.attrs.get("hiopy::nnn", None) != 4:
        logging.info(
            f"Cannot check {source_var.name} to {target_var.name}"
            f", it doesnt use 4-NNN interpolation."
        )
        return
    assert np.allclose(sum([source_var[..., i::4] for i in range(4)]) / 4, target_var)


def check_metadata(var):
    assert (
        "hiopy::copy_metadata" not in var.attrs
    ), f"Attributes of {var.name} {var.attrs.asdict()} is not cleaned"


def _collect_groups(dataset, parent=None):
    if "crs" in dataset.array_keys():
        yield dataset, parent
    for _name, g in dataset.groups():
        yield from _collect_groups(g, dataset)


def main():
    parser = ArgumentParser()
    parser.add_argument("dataset", type=zarr.open, help="Datatset to check")
    args = parser.parse_args()
    for g, p in _collect_groups(args.dataset):
        if "hiopy::parent" in g.attrs:
            parent = p[g.attrs["hiopy::parent"]]
            for name, item in g.arrays():
                if {"cell", "time"} <= set(item.attrs["_ARRAY_DIMENSIONS"]):
                    check_interpolation(parent[name], item)
                    check_metadata(item)


if __name__ == "__main__":
    main()
