#ifndef COYOTE_COYOTE_HPP
#define COYOTE_COYOTE_HPP

#include <functional>
#include <map>
#include <memory>
#include <mpi.h>
#include <optional>
#include <span>
#include <string>

#include <field.hpp>

namespace coyote {

    class Event;

    class Coyote {
        class Impl;
        std::unique_ptr<Impl> pImpl;

      public:
        Coyote(Coyote&&);
        Coyote(const Coyote&) = delete;
        Coyote(const std::string& name);
        Coyote& operator=(Coyote&&);
        Coyote& operator=(const Coyote&) = delete;
        ~Coyote();

        void add_field(
            const std::string& source_comp,
            const std::string& source_grid,
            const std::string& field_name,
            const std::function<void(Event event)>& eventHandler,
            std::optional<std::string> iso_dt            = std::nullopt,
            std::optional<int> collection_size           = std::nullopt,
            std::string interpolation_stack_yaml         = "[nnn]",
            int yac_time_reduction                       = 0,
            std::vector<double> frac_mask                = {},
            std::optional<std::string> pass_through_name = std::nullopt,
            int src_lag                                  = 1,
            std::optional<std::string> weight_filename   = std::nullopt);

        MPI_Comm get_mpi_comm() const;

        // grids
        // yac wrapper
        // wraps yac_cdef_grid_unstructured() and yac_cdef_grid_cloud()
        void def_grid(std::span<const int> num_vertices_per_cell,
                      std::span<const double> x_vertices,
                      std::span<const double> y_vertices,
                      std::span<const int> cell_to_vertex,
                      std::optional<std::span<const double>> x_cells      = std::nullopt,
                      std::optional<std::span<const double>> y_cells      = std::nullopt,
                      std::optional<std::span<const int>> global_cell_idx = std::nullopt);

        // functions to get the necessary info on the available variables
        std::string get_start_datetime() const;
        std::string get_end_datetime() const;

        std::map<std::string, std::map<std::string, std::vector<std::string>>>
        get_all_comp_grid_fields() const;
    };

    void coyote_init(std::string group_name = "", uint nthreads = 1);
    MPI_Comm coyote_group_comm();
    std::string coyote_start_datetime();
    std::string coyote_end_datetime();
    void ensure_enddef();
    int get_field_collection_size(const std::string& source_comp,
                                  const std::string& source_grid,
                                  const std::string& field_name);
    std::string get_field_timestep(const std::string& source_comp,
                                   const std::string& source_grid,
                                   const std::string& field_name);
    std::string get_field_metadata(const std::string& source_comp,
                                   const std::string& source_grid,
                                   const std::string& field_name);
    void coyote_run();

} // namespace coyote
#endif
