#ifndef COYOTE_FIELD_HPP
#define COYOTE_FIELD_HPP

#include <functional>
#include <optional>
#include <semaphore>
#include <vector>

#include <mpi.h>
extern "C" {
#include <yac.h>
}

#include "event.hpp"

namespace coyote {

    class Field {
        friend class Event;

      public:
        Field(const int& target_field_id,
              const std::optional<int>& source_field_id,
              const std::vector<double>& buffer,
              const std::vector<double>& frac_mask,
              const int& collection_size,
              const std::function<void(Event)>& eventHandle);

        bool is_done() const;
        bool is_ready();

        void put_forward();

        std::function<void(Event)> op;
        std::binary_semaphore semaphore = std::binary_semaphore(1);
        std::string datetime() const;

      private:
        int _target_field_id;
        std::optional<int> _source_field_id;
        std::vector<double> _buffer;
        std::vector<double> _frac_mask;
        int _collection_size;
        size_t _point_size;
        int _info         = YAC_ACTION_NONE;
        bool _get_started = false;
        std::string _datetime;
    };
} // namespace coyote

#endif
