#ifndef COYOTE_QUEUE_HPP
#define COYOTE_QUEUE_HPP

#include <condition_variable>
#include <functional>
#include <mutex>
#include <optional>
#include <queue>
#include <thread>

class QueueTerminated {};

template <class T, class P, class Compare = std::less<P>>
class TSQueue {
  private:
    typedef std::optional<std::tuple<P, T>> elem_type;

  public:
    TSQueue() : q(), m(), c() {}

    ~TSQueue() {}

    void enqueue(T t, P prio) {
        std::lock_guard<std::mutex> lock(m);
        q.push(std::make_tuple(prio, t));
        c.notify_one();
    }

    void end() {
        std::lock_guard<std::mutex> lock(m);
        q.push(std::nullopt);
        c.notify_all();
    }

    T dequeue() {
        std::unique_lock<std::mutex> lock(m);
        c.wait(lock, [&] { return !q.empty(); });
        elem_type val = q.top();
        if (!val) {
            throw QueueTerminated();
        }
        q.pop();
        return std::get<1>(val.value());
    }

    bool terminated() {
        std::unique_lock<std::mutex> lock(m);
        if (q.empty())
            return false;
        elem_type val = q.top();
        return (bool)val.value();
    }

    size_t empty() const { return q.empty(); }

  private:
    // only compare the priority and ensure that the "end element" comes last
    struct _Compare {
        Compare compare;
        bool operator()(const elem_type& a, const elem_type& b) const {
            if (!a)
                return true;
            if (!b)
                return false;
            return compare(std::get<0>(a.value()), std::get<0>(b.value()));
        }
    };
    std::priority_queue<elem_type, std::vector<elem_type>, _Compare> q;
    mutable std::mutex m;
    std::condition_variable c;
};

#endif
